Struct readable::Int

source ·
pub struct Int(_, _);
Human readable integer.

From takes an unsigned number as input and returns a ready-to-print!() Int.

f32 or f64 inputs will work, but the fractional parts will be ignored.

Exceptions

To disable checks for these, (you are sure you don’t have NaN’s), enable the ignore_nan_inf feature flag.

Examples

InputOutput
00
11
999999
10001,000
12345671,234,567
100000000100,000,000
1.1231
2000.1232,000

Implementations

source

impl Int

source

pub fn new() -> Self

Returns a Self with the value 0.

source

pub fn unknown() -> Self

Returns a Self with the u64 set to 0, but the String set to ???.

source

pub fn as_str(&self) -> &str

Return a borrowed str without consuming Self.

source

pub fn to_u64(&self) -> u64

Returns the inner u64.

source

pub fn into_string(self) -> String

Consumes [Self], returning the inner String.

source

pub fn into_raw(self) -> (u64, String)

Consumes Self, returning the inner u64 and String.