Function datetime

pub fn datetime() -> (i16, u8, u8, u32)
Available on crate feature time only.
Combines crate::date::date() and time()

This returns the system’s current (year, month, day, seconds_passed_today).

The seconds passed represents how many seconds have passed in the current day, the same as time.