Struct readable::toa::ItoaTmp

source ·
pub struct ItoaTmp { /* private fields */ }
A short-lived version of Itoa

This version doesn’t save the formatting computation, and is meant for cases where the lifetime of the formatted output &str is very brief.

This version has less overhead, but the string must be formatted everytime you need it.

See crate::itoa!() for a quick 1-line format macro.

assert_eq!(ItoaTmp::new().format(10), "10");

You could keep a ItoaTmp around to use it as a factory to keep formatting new strings, as it will reuse the inner buffer:

let mut itoa = ItoaTmp::new();

assert_eq!(itoa.format(10), "10");
assert_eq!(itoa.format(20), "20");
assert_eq!(itoa.format(30), "30");

Size

assert_eq!(std::mem::size_of::<readable::ItoaTmp>(), 40);

Implementations

source

impl ItoaTmp

source

pub const fn new() -> Self

Create a new ItoaTmp.

source

pub fn format<I: Integer>(&mut self, integer: I) -> &str

Format an Integer into a &str with an existing ItoaTmp

// We can cheaply reuse this.
let mut itoa = ItoaTmp::new();

assert_eq!(itoa.format(1),    "1");
assert_eq!(itoa.format(10),   "10");
assert_eq!(itoa.format(100),  "100");
assert_eq!(itoa.format(1000), "1000");