pub struct Int(_, _);Expand description
Human readable integer.
From takes an unsigned number as input and returns a ready-to-print!() Int.
f32 or f64 inputs will work, but the fractional parts will be ignored.
Exceptions
| Exceptions | String Output |
|---|---|
f64::NAN | NaN |
f64::INFINITY & f64::NEG_INFINITY | ∞ |
To disable checks for these, (you are sure you don’t have NaN’s), enable the ignore_nan_inf feature flag.
Examples
| Input | Output |
|---|---|
0 | 0 |
1 | 1 |
999 | 999 |
1000 | 1,000 |
1234567 | 1,234,567 |
100000000 | 100,000,000 |
1.123 | 1 |
2000.123 | 2,000 |
Implementations